@@ -13,6 +13,7 @@ import urllib.parse
 import traceback
 import base64
 import minio
+from urllib.parse import urlparse, parse_qs
 
 import dqueue.core 
 import dqueue.app
@@ -53,9 +54,19 @@ logger=logging.getLogger(__name__)
 
 ## === schemas
 
+class HubVersion(Schema):
+    version = fields.Str()
+
 class TaskData(Schema):
     pass
 
+class SubmissionData(Schema):
+    pass
+
+class TaskPayload(Schema):
+    task_data = TaskData
+    submission_data = SubmissionData
+
 class Task(Schema):
     state = fields.Str()
     queue = fields.Str()
@@ -799,10 +810,10 @@ class WorkerQuestion(SwaggerView):
                     'type': 'string',
                 },
                 {
-                    'name': 'task_data',
+                    'name': 'task_payload',
                     'in': 'body',
                     'required': True,
-                    'schema': TaskData,
+                    'schema': TaskPayload,
                 },
                 {
                     'name': 'queue',
@@ -822,13 +833,14 @@ class WorkerQuestion(SwaggerView):
     def post(self):
         queue = request.args.get('queue', 'default')
         worker_id = request.args.get('worker_id')
-        task_data = request.json
+        task_data = request.json['task_data']
+        submission_data = request.json['submission_data']
 
         queue = dqueue.core.Queue(worker_id=worker_id, queue=queue)
 
         print("got:", worker_id, task_data)
 
-        task_entry = queue.put(task_data)
+        task_entry = queue.put(task_data, submission_data=submission_data)
 
         logger.warning("questioned task: %s", task_entry)
         return jsonify(
@@ -841,6 +853,50 @@ app.add_url_rule(
       methods=['POST']
 )
 
+class HubVersionView(SwaggerView):
+    operationId = "version"
+
+    parameters = [
+                {
+                    'name': 'client_version',
+                    'in': 'query',
+                    'required': False,
+                    'type': 'string',
+                },
+                {
+                    'name': 'worker_id',
+                    'in': 'query',
+                    'required': False,
+                    'type': 'string',
+                },
+            ]
+
+    responses = {
+            200: {
+                    'description': 'hub version info',
+                    'schema': HubVersion,
+                }
+        }
+
+    def get(self):
+        client_version = request.args.get('client_version', None)
+        worker_id = request.args.get('worker_id', None)
+
+        queue = dqueue.core.Queue(worker_id=worker_id)
+        queue.log_task(message=f"client test {client_version} worker {worker_id}", task_key="unset", state="unset")
+
+        version = dqueue.core.__version__
+
+        return jsonify(
+                version=version
+            )
+
+app.add_url_rule(
+         '/hub/version',
+          view_func=HubVersionView.as_view('version'),
+          methods=['GET']
+)
+
 class TaskInfoView(SwaggerView):
     operationId = "task_info"
 
@@ -952,6 +1008,12 @@ class TaskCallbackView(SwaggerView):
     operationId = "callback"
 
     parameters = [
+                {
+                    'name': 'worker_id',
+                    'in': 'query',
+                    'required': True,
+                    'type': 'string',
+                },
                 {
                     'name': 'payload',
                     'in': 'body',
@@ -967,6 +1029,8 @@ class TaskCallbackView(SwaggerView):
         }
 
     def post(self):
+        worker_id = request.args.get('worker_id')
+
         payload = request.json
 
         url = payload['url']
@@ -984,6 +1048,12 @@ class TaskCallbackView(SwaggerView):
         else:
             raise RuntimeError(f"unable to deal with non-standard dispatcher, allowed {allowed_dispatcher}")
 
+        url_parsed = urlparse(url)
+        qs = parse_qs(url_parsed.query)
+        
+        queue = dqueue.core.Queue(worker_id=worker_id)
+        queue.log_task(message=f"callback: {qs.get('job_id', 'unknown')} {params.get('node', 'no-node')}", task_key="unset", state="unset")
+
         return jsonify(
                 dict(
                     status = r.status_code,
