@@ -2,19 +2,20 @@ import click
 import logging
 import json
 import os
+import sys
 import pprint
 import time
 import subprocess
 from termcolor import colored
 
-logger = logging.getLogger()
-
-log = lambda *x,**xx:logger.info(*x, **xx)
-
 from dqueue import from_uri
 from dqueue.core import Queue, Task
 from dqueue.proxy import QueueProxy
 
+import dqueue.core as core 
+
+logger = logging.getLogger()
+
 @click.group()
 @click.option("-q", "--quiet", default=False, is_flag=True)
 @click.option("-d", "--debug", default=False, is_flag=True)
@@ -33,18 +34,41 @@ def cli(obj, quiet=False, debug=False, queue=None):
         queue = os.environ.get('ODAHUB', None)
 
     obj['queue'] = from_uri(queue)
-    logger.info("using queue: %s", obj['queue'])
+    logger.debug("using queue: %s", obj['queue'])
+
+@cli.command()
+@click.option("-v", "--validate", default=False, is_flag=True)
+@click.pass_obj
+def version(obj, validate):
+    hub = obj['queue'].version()
+    print("client:", core.__version__)
+    print("hub:", hub['version'])
+
+    if validate:
+        if core.__version__ == hub['version']:
+            logger.info("versions compatible!")
+        else:
+            logger.error("versions INcompatible!")
+            sys.exit(1)
+
+@cli.command()
+@click.pass_obj
+def auth(obj):
+    print(core.__version__)
+
+
+def log_info(queue):
+    for q in queue.list_queues(None):
+        logger.info(colored(q, 'green'))
+        logger.info("; ".join(
+                f"{k}: {len(v)}" for k,v in q.info.items()
+            ))
+        logger.info("\n")
 
 @cli.command()
 @click.pass_obj
 def info(obj):
-    for q in obj['queue'].list_queues(None):
-        log(colored(q, 'green'))
-        for k,v in q.info.items():
-            print(k, ":", len(v), end="; ")
-        print("\n")
-        #log(q.list(kinds=["waiting","done","failed","running"]))
-        #print(q.show())
+    log_info(obj['queue'])
 
 @cli.command()
 @click.pass_obj
@@ -53,8 +77,8 @@ def purge(obj):
         obj['queue'].purge()
     else:
         for q in Queue().list_queues():
-            log(q.info)
-            log(q.list_task(kinds=["waiting","done","failed","running"]))
+            logger.info(q.info)
+            logger.info(q.list_task(kinds=["waiting","done","failed","running"]))
             q.purge()
 
 def console_size():
@@ -152,6 +176,7 @@ def view(obj, follow, since=0):
                 print("")
                 waiting=False
 
+
             if l['task_key'] not in task_info_cache:
                 ti = obj['queue'].task_by_key(l['task_key'], decode=True)
                 task_info_cache[l['task_key']] = ti
@@ -161,14 +186,18 @@ def view(obj, follow, since=0):
 
             logger.debug(ti)
 
+            name = None
             if ti is not None:
                 try:
                     name = ti['task_dict']['task_data']['object_identity']['factory_name']
                 except Exception as e:
                     logger.error("very stange task: %s; %s", ti.keys(), e)
-                    name = "unnamed"
-            else:
-                name = "unnamed"
+                    name = "missing"
+                    
+            if name is None or name == "??": # ???
+                name = l['message'].split()[-1]
+                l['message'] = l['message'][:-len(name)]
+
 
             print(("{since} {timestamp} "+colored("{task_key:10s}", "red") + " {message:40s} "+colored("{name:20s}", "yellow") + colored(" {worker_id:40s}", "cyan") ).format(
                     since=since,
@@ -302,8 +331,10 @@ def question(obj, task_data):
 def resubmit(obj, scope_selector):
     scope, selector = scope_selector.split(":")
 
+    log_info(obj['queue'])
     r = obj['queue'].resubmit(scope, selector)
     print(colored("resubmitted:", "green"), ":", r)
+    log_info(obj['queue'])
 
 #####
 @cli.group("runner")
